shithub: puzzles

Download patch

ref: a50a65120c3b4230b6a1b95a52c8b6d3299a43c7
parent: 437b69542fc34ae48bee14dc821ef9df1c71c62a
author: Simon Tatham <anakin@pobox.com>
date: Tue May 31 14:38:01 EDT 2005

Better mouse button handling in Mines:
 - middle button now also triggers the clear-around-square action
 - a special-case handler in midend_process_key() arranges that the
   left button always trumps the right button if both are pressed
   together, meaning that Windows Minesweeper players used to
   pressing L+R to clear around a square should still be able to do
   so without any strange behaviour.
(The latter touches all game backends, yet again, to add a field to
the game structure which is zero in everything except Mines.)

[originally from svn r5888]

--- a/cube.c
+++ b/cube.c
@@ -1645,4 +1645,5 @@
     game_flash_length,
     game_wants_statusbar,
     FALSE, game_timing_state,
+    0,				       /* mouse_priorities */
 };
--- a/fifteen.c
+++ b/fifteen.c
@@ -840,4 +840,5 @@
     game_flash_length,
     game_wants_statusbar,
     FALSE, game_timing_state,
+    0,				       /* mouse_priorities */
 };
--- a/midend.c
+++ b/midend.c
@@ -441,6 +441,14 @@
      *    pressed, invent a button-up for the first one and then
      *    pass the button-down through as before.
      * 
+     * 2005-05-31: An addendum to the above. Some games might want
+     * a `priority order' among buttons, such that if one button is
+     * pressed while another is down then a fixed one of the
+     * buttons takes priority no matter what order they're pressed
+     * in. Mines, in particular, wants to treat a left+right click
+     * like a left click for the benefit of users of other
+     * implementations. So the last of the above points is modified
+     * in the presence of an (optional) button priority order.
      */
     if (IS_MOUSE_DRAG(button) || IS_MOUSE_RELEASE(button)) {
         if (me->pressed_mouse_button) {
@@ -454,6 +462,14 @@
         } else
             return ret;                /* ignore it */
     } else if (IS_MOUSE_DOWN(button) && me->pressed_mouse_button) {
+	/*
+	 * If the new button has lower priority than the old one,
+	 * don't bother doing this.
+	 */
+	if (me->ourgame->mouse_priorities &
+	    BUTTON_BEATS(me->pressed_mouse_button, button))
+	    return ret;		       /* just ignore it */
+
         /*
          * Fabricate a button-up for the previously pressed button.
          */
--- a/net.c
+++ b/net.c
@@ -2604,4 +2604,5 @@
     game_flash_length,
     game_wants_statusbar,
     FALSE, game_timing_state,
+    0,				       /* mouse_priorities */
 };
--- a/netslide.c
+++ b/netslide.c
@@ -1762,4 +1762,5 @@
     game_flash_length,
     game_wants_statusbar,
     FALSE, game_timing_state,
+    0,				       /* mouse_priorities */
 };
--- a/nullgame.c
+++ b/nullgame.c
@@ -254,4 +254,5 @@
     game_flash_length,
     game_wants_statusbar,
     FALSE, game_timing_state,
+    0,				       /* mouse_priorities */
 };
--- a/pattern.c
+++ b/pattern.c
@@ -1143,6 +1143,7 @@
     game_flash_length,
     game_wants_statusbar,
     FALSE, game_timing_state,
+    0,				       /* mouse_priorities */
 };
 
 #ifdef STANDALONE_SOLVER
--- a/puzzles.h
+++ b/puzzles.h
@@ -47,6 +47,9 @@
 #define IS_MOUSE_RELEASE(m) ( (unsigned)((m) - LEFT_RELEASE) <= \
                                (unsigned)(RIGHT_RELEASE - LEFT_RELEASE))
 
+/* Bit flags indicating mouse button priorities */
+#define BUTTON_BEATS(x,y) ( 1 << (((x)-LEFT_BUTTON)*3+(y)-LEFT_BUTTON) )
+
 #define IGNOREARG(x) ( (x) = (x) )
 
 typedef struct frontend frontend;
@@ -245,6 +248,7 @@
     int (*wants_statusbar)(void);
     int is_timed;
     int (*timing_state)(game_state *state);
+    int mouse_priorities;
 };
 
 /*
--- a/rect.c
+++ b/rect.c
@@ -2553,4 +2553,5 @@
     game_flash_length,
     game_wants_statusbar,
     FALSE, game_timing_state,
+    0,				       /* mouse_priorities */
 };
--- a/sixteen.c
+++ b/sixteen.c
@@ -1011,4 +1011,5 @@
     game_flash_length,
     game_wants_statusbar,
     FALSE, game_timing_state,
+    0,				       /* mouse_priorities */
 };
--- a/solo.c
+++ b/solo.c
@@ -2179,6 +2179,7 @@
     game_flash_length,
     game_wants_statusbar,
     FALSE, game_timing_state,
+    0,				       /* mouse_priorities */
 };
 
 #ifdef STANDALONE_SOLVER
--- a/twiddle.c
+++ b/twiddle.c
@@ -1179,4 +1179,5 @@
     game_flash_length,
     game_wants_statusbar,
     FALSE, game_timing_state,
+    0,				       /* mouse_priorities */
 };
--- a/mines.c
+++ b/mines.c
@@ -2457,7 +2457,8 @@
     if (cx < 0 || cx >= from->w || cy < 0 || cy > from->h)
 	return NULL;
 
-    if (button == LEFT_BUTTON || button == LEFT_DRAG) {
+    if (button == LEFT_BUTTON || button == LEFT_DRAG ||
+	button == MIDDLE_BUTTON || button == MIDDLE_DRAG) {
 	/*
 	 * Mouse-downs and mouse-drags just cause highlighting
 	 * updates.
@@ -2487,7 +2488,7 @@
 	return ret;
     }
 
-    if (button == LEFT_RELEASE) {
+    if (button == LEFT_RELEASE || button == MIDDLE_RELEASE) {
 	ui->hx = ui->hy = -1;
 	ui->hradius = 0;
 
@@ -2501,8 +2502,9 @@
 	 * permitted if the tile is marked as a mine, for safety.
 	 * (Unmark it and _then_ open it.)
 	 */
-	if (from->grid[cy * from->w + cx] == -2 ||
-	    from->grid[cy * from->w + cx] == -3) {
+	if (button == LEFT_RELEASE &&
+	    (from->grid[cy * from->w + cx] == -2 ||
+	     from->grid[cy * from->w + cx] == -3)) {
 	    ret = dup_game(from);
             ret->just_used_solve = FALSE;
 	    open_square(ret, cx, cy);
@@ -2512,10 +2514,10 @@
 	}
 
 	/*
 	 * Left-clicking on an uncovered tile: first we check to see if
 	 * the number of mine markers surrounding the tile is equal to
 	 * its mine count, and if so then we open all other surrounding
 	 * squares.
 	 */
 	if (from->grid[cy * from->w + cx] > 0) {
 	    int dy, dx, n;
@@ -3006,4 +3008,5 @@
     game_flash_length,
     game_wants_statusbar,
     TRUE, game_timing_state,
+    BUTTON_BEATS(LEFT_BUTTON, RIGHT_BUTTON),
 };
@@ -2512,10 +2514,10 @@
 
 	/*
-	 * Left-clicking on an uncovered tile: first we check to see if
-	 * the number of mine markers surrounding the tile is equal to
-	 * its mine count, and if so then we open all other surrounding
-	 * squares.
+	 * Left-clicking or middle-clicking on an uncovered tile:
+	 * first we check to see if the number of mine markers
+	 * surrounding the tile is equal to its mine count, and if
+	 * so then we open all other surrounding squares.
 	 */
 	if (from->grid[cy * from->w + cx] > 0) {